function point(ccref) {
  return ccref.isPoint();
}

function line(ccref) {
  return ccref.isLine();
}

function notLine(ccref) {
  return !line(ccref) && any(ccref);
}

function arc(ccref) {
  return ccref.isArc() || ccref.isCircle();
}

function curve(ccref) {
  return line(ccref) || arc(ccref);
}

function any(ccref) {
  return point(ccref) || curve(ccref);
}

function numberMoreOrEqual(iPredicate, entityNumber) {
  return {
    entityNumber: entityNumber,
    //TODO: return just check function ??
    checkFunc: function checkFunc(ccrefs, startObj) {
      var curRef = ccrefs[startObj.start];
      var passedRef = 0;

      while (curRef && iPredicate(curRef)) {
        ++passedRef;
        curRef = ccrefs[startObj.start + passedRef];
      }

      if (passedRef >= entityNumber) {
        startObj.start += passedRef;
        return true;
      }

      return false;
    }
  };
}

function f(type, predicates, desc) {
  var isLenient = desc.includes('l'); // allow selecting a line instead of its two endpoints

  var isStrictlyLenient = desc.includes('L'); // must select lines instead of endpoints

  return {
    type: type,
    isParametric: desc.includes('p'),
    isCommutative: desc.includes('c'),
    adapt: function adapt(entities) {
      entities = entities.concat();

      for (var i = 0; i < entities.length; ++i) {
        var _entities;

        if (isLenient && entities[i].isLine() && entities.length < predicates.length) (_entities = entities).splice.apply(_entities, [i, 1].concat(entities[i].children));
      }

      var entityIdx = 0;

      for (var _i = 0; _i < predicates.length; ++_i) {
        if (entityIdx >= entities.length) return null;
        var predicate = predicates[_i];

        if (typeof predicate === 'function') {
          var entity = entities[entityIdx++];
          if (entity === undefined || !predicate(entity)) return null;
        } else if (typeof predicate === 'object') {
          // const entityNumber = predicate.number
          var startObj = {
            start: entityIdx // Pass number by reference

          };
          if (!predicate.checkFunc(entities, startObj)) return null;
          entityIdx += startObj.start;
        }
      }

      for (var _i2 = 0; _i2 < entities.length; ++_i2) {
        var _entities2;

        if (isStrictlyLenient && entities[_i2].isLine()) (_entities2 = entities).splice.apply(_entities2, [_i2, 1].concat(entities[_i2].children));
      }

      return entities;
    }
  };
}

export var angle = f('CC_2DAngleConstraint', [line, line], 'pc');
export var angleox = f('CC_2DAngleOXConstraint', [line], 'p');
export var colinear = f('CC_2DColinearConstraint', [line, line], 'c');
export var concentric = f('CC_2DConcentricConstraint', [arc, arc], 'c');
export var distance = f('CC_2DOffsetConstraint', [any, any], 'pcl');
export var equalDistance = f('CC_2DEqualLengthConstraint', [line, line], 'L');
export var equalRadius = f('CC_2DEqualRadiusConstraint', [arc, arc], 'c');
export var fixation = f('CC_2DFixationConstraint', [any], '');
export var horizontalDistance = f('CC_2DHorizontalDistanceConstraint', [point, point], 'pl');
export var horizontality = f('CC_2DHorizontalConstraint', [point, point], 'l');
export var incidence = f('CC_2DCoincidentConstraint', [notLine, notLine], 'c');
export var midpoint = f('CC_2DMidpointConstraint', [any, point, any], '');
export var parallelity = f('CC_2DParallelConstraint', [line, line], 'c');
export var perpendicularity = f('CC_2DPerpendicularConstraint', [line, line], 'c');
export var radius = f('CC_2DRadiusConstraint', [arc], 'p');
export var diameter = f('CC_2DDiameterConstraint', [arc], 'p');
export var symmetric = f('CC_2DSymmetryConstraint', [line, any, any], '');
export var tangency = f('CC_2DTangentSketchConstraint', [arc, curve], 'c');
export var verticalDistance = f('CC_2DVerticalDistanceConstraint', [point, point], 'pl');
export var verticality = f('CC_2DVerticalConstraint', [point, point], 'l');
export var rigidSet = f('CC_RigidSet', [numberMoreOrEqual(any, 2)], '');
export var use = f('CC_Use', [any], '');