import _inheritsLoose from "@babel/runtime/helpers/inheritsLoose";
import * as THREE from 'three';
import Object3 from '../../../three/object3';
import BaseHandler from './base';
import Ccref from '../ccref';
var useRefInfo = undefined;

var HoverHandler =
/*#__PURE__*/
function (_BaseHandler) {
  _inheritsLoose(HoverHandler, _BaseHandler);

  function HoverHandler(sketcher, name) {
    return _BaseHandler.call(this, sketcher, name) || this; //this.sketcher.selector.activate(this.sketcher.namedElements.selection.types);
  }

  var _proto = HoverHandler.prototype;

  _proto.destroy = function destroy() {
    this.sketcher.selector.unselectAll();
    /*for (let obj of this.sketcher.graphics.values())
            this.sketcher.selector.unhover(obj.localMesh || obj);
        this.sketcher.selector.deactivate();*/

    _BaseHandler.prototype.destroy.call(this);
  };

  _proto[Object3.Events.Interaction.Hovered] = function (ccref, hitObject) {
    if (!(ccref instanceof Ccref)) return; //TODO: remove "object instanceof Ccref" after implementing global Ccref.

    var object = hitObject.object,
        material = hitObject.material,
        first = hitObject.first;
    this.sketcher.constraintVisualizer.hover(ccref, first);
    if (!first) return;
    this.sketcher.selector.hovered({
      object: object,
      material: material
    });

    if (ccref.isConstraint() || ccref.isRigidSet()) {
      for (var _iterator = ccref.mergedEntities, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
        var _ref;

        if (_isArray) {
          if (_i >= _iterator.length) break;
          _ref = _iterator[_i++];
        } else {
          _i = _iterator.next();
          if (_i.done) break;
          _ref = _i.value;
        }

        var entity = _ref;

        if (entity.graphics) {
          this.sketcher.selector.hovered({
            object: entity.graphics.localMesh,
            material: entity.graphics.localMesh.material
          });
        }
      }
    } else if (ccref.isUse()) {
      // Highlight object on sketch.
      var sketchObj = ccref.entities[0];

      if (sketchObj.graphics) {
        this.sketcher.selector.hovered({
          object: sketchObj.graphics.localMesh,
          material: sketchObj.graphics.localMesh.material
        });
      } // Highlight origin edge or point.


      var refObjId = ccref.entities[1].id;
      var refObjMembers = this.sketcher.tree[refObjId].members;
      var refType = refObjMembers.edgeId !== undefined ? 'edge' : 'point';
      var referencedId;

      if (refType === 'edge') {
        referencedId = refObjMembers.edgeId.value;
        var refObjGraphics = this.sketcher.session.pool.findMaterial(function (material) {
          return material.meta && material.meta.id === referencedId;
        });

        if (refObjGraphics) {
          this.sketcher.selector.hovered({
            object: refObjGraphics.object,
            material: refObjGraphics.material
          });
        }

        useRefInfo = {
          type: 'edge',
          graphics: refObjGraphics
        };
      } else if (refType === 'point') {
        var _refObjMembers$vertex;

        referencedId = (_refObjMembers$vertex = refObjMembers.vertexId) === null || _refObjMembers$vertex === void 0 ? void 0 : _refObjMembers$vertex.value;
        var point;
        var refObjRegion = this.sketcher.session.pool.find(function (obj) {
          if (obj.type === 'Region') {
            point = obj.points.find(function (p) {
              return p.meta.id === referencedId;
            });
            return point !== undefined;
          }

          return false;
        });
        refObjRegion.createPoint(point);
        this.sketcher.session.pool.add(point.meta.ref); //TODO: remove this hack. Should work with only refObjRegion.createPoint()

        this.sketcher.selector.hovered({
          object: refObjRegion,
          material: point.meta.ref.material
        });
        useRefInfo = {
          type: 'point',
          graphics: point.meta.ref
        };
      }
    }
  };

  _proto[Object3.Events.Interaction.Unhovered] = function (ccref, hitObject) {
    if (!(ccref instanceof Ccref)) return; //TODO: remove "object instanceof Ccref" after implementing global Ccref.

    var object = hitObject.object,
        material = hitObject.material,
        first = hitObject.first;
    this.sketcher.constraintVisualizer.unhover(ccref, first);
    this.sketcher.selector.unhovered({
      object: object,
      material: material
    });

    if (ccref.isConstraint() || ccref.isRigidSet()) {
      for (var _iterator2 = ccref.mergedEntities, _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _iterator2[Symbol.iterator]();;) {
        var _ref2;

        if (_isArray2) {
          if (_i2 >= _iterator2.length) break;
          _ref2 = _iterator2[_i2++];
        } else {
          _i2 = _iterator2.next();
          if (_i2.done) break;
          _ref2 = _i2.value;
        }

        var entity = _ref2;

        if (entity.graphics) {
          this.sketcher.selector.unhovered({
            object: entity.graphics.localMesh,
            material: entity.graphics.localMesh.material
          });
        }
      }
    } else if (ccref.isUse()) {
      var sketchObj = ccref.entities[0];

      if (sketchObj.graphics) {
        this.sketcher.selector.unhovered({
          object: sketchObj.graphics.localMesh,
          material: sketchObj.graphics.localMesh.material
        });
      }

      var _useRefInfo = useRefInfo,
          type = _useRefInfo.type,
          graphics = _useRefInfo.graphics;

      if (type === 'point') {
        this.sketcher.session.pool.remove(graphics);
      } else if (type === 'edge') {
        this.sketcher.selector.unhovered({
          object: graphics.object,
          material: graphics.material
        });
      }

      useRefInfo = {};
    }
  };

  return HoverHandler;
}(BaseHandler);

export { HoverHandler as default };