import _objectSpread from "@babel/runtime/helpers/objectSpread";
import { getClass } from './geomutils';
import { Graphics } from './graphics';

function changeGeomObjectContainerType(container) {
  for (var _len = arguments.length, geomTypes = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    geomTypes[_key - 1] = arguments[_key];
  }

  if (container.geomType === geomTypes[0]) return;
  container.geomType = geomTypes[0];

  while (container.children.length > 0) {
    container.remove(container.children[container.children.length - 1]);
  }

  for (var _i = 0; _i < geomTypes.length; _i++) {
    var geomType = geomTypes[_i];
    container.add(Graphics(geomType));
  }
} // update object and its start, end and center point objects


export function updateGeomObjectContainer(container, geomParams) {
  switch (getClass(geomParams)) {
    case 'CC_Point':
      changeGeomObjectContainerType(container, 'CC_Point');
      container.children[0].updateFromGeomParams(geomParams);
      break;

    case 'CC_Line':
      changeGeomObjectContainerType(container, 'CC_Line', 'CC_Point', 'CC_Point');
      container.children[0].updateFromGeomParams(geomParams);
      container.children[1].updateFromGeomParams(_objectSpread({}, geomParams, {
        start: geomParams.start
      }));
      container.children[2].updateFromGeomParams(_objectSpread({}, geomParams, {
        start: geomParams.end
      }));
      break;

    case 'CC_Arc':
      changeGeomObjectContainerType(container, 'CC_Arc', 'CC_Point', 'CC_Point', 'CC_Point');
      container.children[0].updateFromGeomParams(geomParams);
      container.children[1].updateFromGeomParams(_objectSpread({}, geomParams, {
        start: geomParams.start
      }));
      container.children[2].updateFromGeomParams(_objectSpread({}, geomParams, {
        start: geomParams.end
      }));
      container.children[3].updateFromGeomParams(_objectSpread({}, geomParams, {
        start: geomParams.center
      }));
      break;

    case 'CC_Circle':
      changeGeomObjectContainerType(container, 'CC_Circle', 'CC_Point');
      container.children[0].updateFromGeomParams(geomParams);
      container.children[1].updateFromGeomParams(_objectSpread({}, geomParams, {
        start: geomParams.center
      }));
      break;

    case 'Rectangle':
      changeGeomObjectContainerType(container, 'CC_Line', 'CC_Line', 'CC_Line', 'CC_Line', 'CC_Point', 'CC_Point', 'CC_Point', 'CC_Point');
      container.children[0].updateFromGeomParams(_objectSpread({}, geomParams, {
        start: geomParams.p00,
        end: geomParams.p10
      }));
      container.children[1].updateFromGeomParams(_objectSpread({}, geomParams, {
        start: geomParams.p00,
        end: geomParams.p01
      }));
      container.children[2].updateFromGeomParams(_objectSpread({}, geomParams, {
        start: geomParams.p10,
        end: geomParams.p11
      }));
      container.children[3].updateFromGeomParams(_objectSpread({}, geomParams, {
        start: geomParams.p01,
        end: geomParams.p11
      }));
      container.children[4].updateFromGeomParams(_objectSpread({}, geomParams, {
        start: geomParams.p00
      }));
      container.children[5].updateFromGeomParams(_objectSpread({}, geomParams, {
        start: geomParams.p01
      }));
      container.children[6].updateFromGeomParams(_objectSpread({}, geomParams, {
        start: geomParams.p10
      }));
      container.children[7].updateFromGeomParams(_objectSpread({}, geomParams, {
        start: geomParams.p11
      }));
      break;

    default:
      changeGeomObjectContainerType(container);
      break;
  }
}