import _objectSpread from "@babel/runtime/helpers/objectSpread";
import _createClass from "@babel/runtime/helpers/createClass";
import _inheritsLoose from "@babel/runtime/helpers/inheritsLoose";
import _assertThisInitialized from "@babel/runtime/helpers/assertThisInitialized";
import * as THREE from 'three';
import uniq from 'lodash/uniq';
import merge from 'lodash/merge';
import isEqual from 'lodash/isEqual';
import Events from '../core/events';
import Region from '../three/region';
import Hud from '../core/hud';
import CombinedCamera from '../three/combinedcamera';
var HOVERED = THREE.Object3D.Events.Interaction.Hovered;
var UNHOVERED = THREE.Object3D.Events.Interaction.Unhovered;
var CLICKED = THREE.Object3D.Events.Interaction.Clicked;
var MISSED = THREE.Object3D.Events.Interaction.Missed;
var PICKED = THREE.Object3D.Events.Interaction.Picked;
var DROPPED = THREE.Object3D.Events.Interaction.Dropped;
var DRAGGED = THREE.Object3D.Events.Interaction.Dragged;
var RENDERED = THREE.Object3D.Events.Lifecycle.Rendered;
var EPSILON = 1e-4;

var Selector =
/*#__PURE__*/
function (_Events) {
  _inheritsLoose(Selector, _Events);

  function Selector(session, options) {
    var _this;

    _this = _Events.call(this) || this;

    _this._unselectChild = function (child) {
      if (child.material) {
        var props = child.mapMaterial(function (material) {
          return material.meta && (material.meta.clicked = undefined);
        });
        child.animate(props).start(_this.options.duration);
      }
    };

    _this.hovered = function (_ref) {
      var _properties$object$ty;

      var object = _ref.object,
          material = _ref.material;
      var _this$options = _this.options,
          target = _this$options.target,
          properties = _this$options.properties,
          duration = _this$options.duration;
      if (_this.interfaceSelectors.length) Array.from(_this.map).forEach(function (_ref2) {
        var child = _ref2[0],
            interfaces = _ref2[1];
        var props = interfaces.reduce(function (prev, selector) {
          var parent = object.findReverse(function (parent) {
            return parent === selector;
          });
          var result = merge(prev, child.mapMaterial(function (cM) {
            var _properties;

            return parent && cM.meta ? _objectSpread({}, (_properties = properties[cM === material ? child.type : selector.type]) === null || _properties === void 0 ? void 0 : _properties[HOVERED]) : null;
          }));
          return result;
        }, {});
        child.animate(props).start(duration);
      });else material.meta && material.animate((_properties$object$ty = properties[object.type]) === null || _properties$object$ty === void 0 ? void 0 : _properties$object$ty[HOVERED]).start(0);

      _this.emit(HOVERED, {
        object: object,
        material: material
      });

      return _assertThisInitialized(_assertThisInitialized(_this));
    };

    _this.unhovered = function (_ref3) {
      var object = _ref3.object,
          material = _ref3.material;
      var _this$options2 = _this.options,
          target = _this$options2.target,
          duration = _this$options2.duration;
      if (_this.interfaceSelectors.length) Array.from(_this.map).forEach(function (_ref4) {
        var child = _ref4[0],
            interfaces = _ref4[1];
        var props = child.mapMaterial(function (childMat) {
          return childMat.meta && childMat.meta.clicked;
        });
        child.animate(props).start(duration);
      });else material.meta && material.animate(_objectSpread({}, material.meta.material, material.meta.clicked)).start(duration);

      _this.emit(UNHOVERED, {
        object: object,
        material: material
      });

      return _assertThisInitialized(_assertThisInitialized(_this));
    };

    _this.clicked = function (props) {
      var object = props.object,
          material = props.material;
      var _this$options3 = _this.options,
          limit = _this$options3.limit,
          properties = _this$options3.properties,
          types = _this$options3.types,
          duration = _this$options3.duration,
          selectObjects = _this$options3.selectObjects;

      var id = _this.getHitId(props); // material.meta.id
      //Some objects shouldn't be highlighted, so check .meta in material before ("none-selectable" objects have material without .meta).


      if (material.meta && _this.selected.has(id)) return _this.unselectAll(true).remove(id).selectAll();
      if (_this.selected.size >= limit) _this.unselectAll(true).remove(_this.last).selectAll();

      if (_this.interfaceSelectors.length) {
        Array.from(_this.map).forEach(function (_ref5, index) {
          var child = _ref5[0],
              interfaces = _ref5[1];
          var props = interfaces.reduce(function (prev, selector) {
            var objectParent = object.findReverse(function (parent) {
              return parent === selector;
            });
            var next = child.mapMaterial(function (childMat) {
              var _properties2;

              if (!objectParent || !childMat.meta) return null;

              var previous = _this.selected.get(childMat.meta.id);

              var clicked = childMat === material || previous !== undefined;

              if (childMat === material) {
                _this.add(id, {
                  targets: uniq((previous ? previous.targets : []).concat([selector])),
                  object: object,
                  material: material
                });
              }

              return childMat.meta.clicked = _objectSpread({}, childMat.meta.clicked, (_properties2 = properties[clicked ? child.type : selector.type]) === null || _properties2 === void 0 ? void 0 : _properties2[CLICKED]);
            });
            return merge(prev, next);
          }, {});
          child.animate(props).start(duration);
        });
      } else if (material.meta && properties[object.type]) {
        var _properties$object$ty2;

        _this.add(id, {
          object: object,
          material: material
        });

        material.animate(material.meta.clicked = (_properties$object$ty2 = properties[object.type]) === null || _properties$object$ty2 === void 0 ? void 0 : _properties$object$ty2[CLICKED]).start(duration);
      }

      _this.emit(CLICKED, {
        object: object,
        material: material
      });

      return _assertThisInitialized(_assertThisInitialized(_this));
    };

    _this.missed = function () {
      if (_this.selectedIds.length === 0) return _assertThisInitialized(_assertThisInitialized(_this)); //avoid traversing whole scene per each object with missed callback

      _this.unselectAll();

      _this.emit(MISSED);

      return _assertThisInitialized(_assertThisInitialized(_this));
    };

    _this.rendered = function () {
      _this.calculateLinePrecision();

      _this.emit(RENDERED);

      return _assertThisInitialized(_assertThisInitialized(_this));
    };

    _this.handleCenterPointSelection = function (_ref6) {
      var _material$meta;

      var object = _ref6.object,
          material = _ref6.material;

      switch (material === null || material === void 0 ? void 0 : (_material$meta = material.meta) === null || _material$meta === void 0 ? void 0 : _material$meta.type) {
        // find (or create) a center point inside region when a circle/arc is selected and make it visible
        case 'circle':
        case 'arc':
          var cPos = material.meta.center;

          _this.regions.forEach(function (region) {
            region.points.forEach(function (point, index) {
              if (region.parent === object.parent) {
                var _pos = point.meta.position;

                if (Math.abs(cPos.x - _pos.x) < EPSILON && Math.abs(cPos.y - _pos.y) < EPSILON && Math.abs(cPos.z - _pos.z) < EPSILON) {
                  if (!point.meta.ref) {
                    region.createPoint(point);
                  }

                  point.meta.ref.material.animate({
                    opacity: 0.3
                  }).start(0);
                }
              }
            });
          });

          break;
        // unselect all circle/arc when a center point is selected

        case 'point':
          var pos = material.meta.position;

          _this.selected.forEach(function (_ref7) {
            var object = _ref7.object,
                material = _ref7.material;

            if (['circle', 'arc'].includes(material.meta.type)) {
              var _cPos = material.meta.center;

              if (Math.abs(_cPos.x - pos.x) < EPSILON && Math.abs(_cPos.y - pos.y) < EPSILON && Math.abs(_cPos.z - pos.z) < EPSILON) {
                var id = _this.getHitId({
                  object: object,
                  material: material
                });

                _this.unselectAll(true).remove(id).selectAll();
              }
            }
          });

          break;
      }
    };

    _this.session = session;
    _this.last = undefined;
    _this.map = new Map();
    _this.selected = new Map();
    _this.unselectedObjects = new Map();
    _this.options = _objectSpread({
      ids: [],
      target: session.pool,
      limit: Infinity,
      types: ['Mesh'],
      whitelist: ['Region'],
      pierce: ['Assembly', 'Part', 'Model', 'Workplane'],
      properties: session.globals.selection,
      shift: true,
      duration: 0,
      recursive: true,
      first: true,
      selectObjects: false,
      priority: 0,
      createInteraction: true
    }, options); // Allow undefined target, this will keep selector from creating a interaction-handler

    if (!_this.options.target) _this.options.target = new THREE.Object3D();
    var _this$options4 = _this.options,
        ids = _this$options4.ids,
        pierce = _this$options4.pierce,
        _types = _this$options4.types,
        recursive = _this$options4.recursive,
        whitelist = _this$options4.whitelist,
        _target = _this$options4.target,
        first = _this$options4.first,
        priority = _this$options4.priority,
        _limit = _this$options4.limit,
        shift = _this$options4.shift,
        createInteraction = _this$options4.createInteraction,
        _selectObjects = _this$options4.selectObjects,
        _duration = _this$options4.duration; // Save original target

    _this.originalTarget = _target; // Find object.type grouping interfaces that aren't real THREE types (f.i. Assembly, Part, etc.)

    _this.interfaceSelectors = _types.filter(function (t) {
      return whitelist.indexOf(t) == -1 && (!THREE[t] || !new THREE[t]().material);
    });
    _this.basicSelectors = _this.options.basic || _types.filter(function (t) {
      return _this.interfaceSelectors.indexOf(t) === -1;
    });

    _target.viewFound().then(function (view) {
      var _target$createInterac;

      _this.view = view;
      _this.regions = [];

      if (_types.indexOf('Region') >= 0) {
        // Only activate reagions when the selectors explicitely asks for it
        _target.traverse(function (child) {
          return child.type === 'Region' && _this.regions.push(child);
        });

        _this.regions.forEach(function (region) {
          return region.visible = true;
        });
      }

      if (createInteraction) _target.createInteraction({
        pierce: pierce,
        types: _types,
        recursive: recursive,
        first: first,
        priority: priority
      }).on((_target$createInterac = {}, _target$createInterac[HOVERED] = _this.hovered, _target$createInterac[UNHOVERED] = _this.unhovered, _target$createInterac[CLICKED] = _this.clicked, _target$createInterac[MISSED] = _this.missed, _target$createInterac[RENDERED] = _this.rendered, _target$createInterac[PICKED] = function () {
        var _this2;

        for (var _len = arguments.length, props = new Array(_len), _key = 0; _key < _len; _key++) {
          props[_key] = arguments[_key];
        }

        return (_this2 = _this).emit.apply(_this2, [PICKED].concat(props));
      }, _target$createInterac[DRAGGED] = function () {
        var _this3;

        for (var _len2 = arguments.length, props = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
          props[_key2] = arguments[_key2];
        }

        return (_this3 = _this).emit.apply(_this3, [DRAGGED].concat(props));
      }, _target$createInterac[DROPPED] = function () {
        var _this4;

        for (var _len3 = arguments.length, props = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
          props[_key3] = arguments[_key3];
        }

        return (_this4 = _this).emit.apply(_this4, [DROPPED].concat(props));
      }, _target$createInterac)); // Pre-select objects from ids

      if (ids.length > 0) {
        _this.buildSelected(ids).selectAll();
      }

      var oldState = _this.selectedIds;

      _this.update();

      _this.on(['add', 'remove', 'removeAll'], function () {
        var state = _this.selectedIds;
        !isEqual(state, oldState) && _this.emit('changed', state, oldState);
        oldState = state;
      });

      if (shift) {
        // SHIFT KEY selection handler
        _this.shiftHandler = function (event) {
          var candidates = [],
              oldIds = [];

          if (!_this.shiftHandler.handled && event.keyCode === 16) {
            _this.shiftHandler.handled = true;
            _this.view.input.debounce = false;
            _this.view.controls.enabled = false;
            _this.view.interaction.enabled = false;

            _target.traverseMaterials(function (material, object) {
              if (object.visible && (object.type === 'RegionPoint' || object.interactive) && material.meta && material.visible) {
                // project bounding box to view coordinates
                object.updateMatrixWorld(true); //TODO: get and project all 8 points and not only min and max

                var minPt, maxPt;

                if (!material.meta.box) {
                  !object.geometry.boundingBox && object.geometry.computeBoundingBox();
                  minPt = object.geometry.boundingBox.min.clone();
                  maxPt = object.geometry.boundingBox.max.clone();
                } else {
                  minPt = material.meta.box.min.clone();
                  maxPt = material.meta.box.max.clone();
                } // project to view coordinates


                var p1 = _this.view.getPoint2(minPt);

                var p2 = _this.view.getPoint2(maxPt); // bounding box in 2D


                var bounds = new THREE.Box2();
                bounds.setFromPoints([p1, p2]);
                candidates.push({
                  object: object,
                  material: material,
                  bounds: bounds
                });
              }
            }); // init new heads up display, add it to the view


            _this.hud = new Hud(_this.view);

            _this.view.addHud(_this.hud);

            _this.hud.controls = undefined;
            _this.hud.camera = new CombinedCamera({
              near: _this.view.camera.near,
              far: _this.view.camera.far,
              fov: _this.view.camera.fov
            });

            _this.view.measure(true);

            var geom = new THREE.PlaneGeometry(0.001, 0.001);
            var box = new THREE.Mesh(geom, new THREE.MeshBasicMaterial({
              transparent: true,
              opacity: 0.05,
              color: new THREE.Color(0)
            }));

            _this.hud.scene.add(box); // set up mouse action's handler


            var clickDown, mousePos, rubberBandLine, clickHud;

            var handler = function handler(event) {
              switch (event.type) {
                case 'mousedown':
                  clickDown = new THREE.Vector2(event.offsetX, event.offsetY);
                  mousePos = new THREE.Vector2(event.offsetX + 1, event.offsetY + 1);
                  clickHud = _this.view.getPoint3({
                    x: event.offsetX,
                    y: event.offsetY
                  }, _this.hud.camera.display, 0);
                  box.position.copy(clickHud);
                  break;

                case 'mousemove':
                  if (!clickDown) {
                    return;
                  } // update rectangle


                  var tempPoint = _this.view.getPoint3({
                    x: event.offsetX,
                    y: event.offsetY
                  }, _this.hud.camera.display, 0);

                  var delta = tempPoint.sub(clickHud);
                  var width = Math.abs(delta.x);
                  var height = Math.abs(delta.y);
                  var halfHeight = height / 2;
                  var halfWidth = width / 2;
                  var halfDeltaX = delta.x / 2;
                  var halfDeltaY = delta.y / 2;
                  geom.vertices[0].set(-halfWidth + halfDeltaX, halfHeight + halfDeltaY, 0);
                  geom.vertices[1].set(width - halfWidth + halfDeltaX, halfHeight + halfDeltaY, 0);
                  geom.vertices[2].set(-halfWidth + halfDeltaX, -(height - halfHeight) + halfDeltaY, 0);
                  geom.vertices[3].set(width - halfWidth + halfDeltaX, -(height - halfHeight) + halfDeltaY, 0);
                  geom.verticesNeedUpdate = true;
                  mousePos = new THREE.Vector2(event.offsetX, event.offsetY); // check intersection with rectangle in display-coordinates

                  var rubberBandRectangle = new THREE.Box2();
                  rubberBandRectangle.setFromPoints([clickDown, mousePos]);
                  var _ids = [],
                      id = 0;

                  for (var _i = 0; _i < candidates.length; _i++) {
                    var sel = candidates[_i];
                    id = _this.getHitId(sel);

                    if (_ids.indexOf(id) === -1 && rubberBandRectangle.containsBox(sel.bounds)) {
                      _ids.push(id);

                      if (_ids.length >= _limit) break;
                    }
                  }

                  if (!isEqual(_ids, oldIds)) {
                    oldIds = _ids;

                    _this.unselectAll(true).buildSelected(_ids).selectAll();
                  }

                  _this.view.invalidate();

                  break;

                case 'mouseup':
                  box && box.destroy();
                  clickDown = undefined;

                  if (rubberBandLine !== undefined) {
                    _this.view.scene.remove(rubberBandLine);

                    rubberBandLine = undefined;
                  }

                  break;
              }
            };

            _this.view.input.on(['mousedown', 'mousemove', 'mouseup'], handler);

            _this.tempHandler = function () {
              // remove and destroy heads up display
              _this.view.removeHud(_this.hud);

              _this.hud.destroy();

              _this.shiftHandler.handled = false;

              _this.view.input.removeListener(['mousedown', 'mousemove', 'mouseup'], handler);

              document.removeEventListener('keyup', _this.tempHandler);
              _this.view.input.debounce = true;
              _this.view.controls.enabled = true;
              _this.view.interaction.enabled = true;
            };

            document.addEventListener('keyup', _this.tempHandler);
          }
        };

        _this.shiftHandler.handled = false;
        document.addEventListener('keydown', _this.shiftHandler);
      }

      _this.emit('created');
    });

    return _this;
  }

  var _proto = Selector.prototype;

  _proto.destroy = function destroy() {
    this.tempHandler && this.tempHandler();
    this.tempHandler = undefined;
    this.shiftHandler && document.removeEventListener('keydown', this.shiftHandler);
    this.shiftHandler = undefined;
    this.regions && this.regions.forEach(function (region) {
      return region.visible = false;
    });
    this.unselectAll(true).removeAll(false);
    this.emit('destroy');
    this.removeListeners();
    this.options.target.removeInteraction();
    this.session = undefined;
    this.last = undefined;
    this.selected = undefined;
    this.options = undefined;
    this.interfaceSelectors = undefined;
  }; // Creates the internal map (this.selected) from an array of string id's


  _proto.buildSelected = function buildSelected(ids) {
    var _this5 = this;

    var _this$options5 = this.options,
        target = _this$options5.target,
        properties = _this$options5.properties,
        types = _this$options5.types,
        selectObjects = _this$options5.selectObjects;
    this.selected.clear();
    if (!ids.length) this.removeAll();

    if (selectObjects) {
      ids.forEach(function (id) {
        var object = target.find(function (object) {
          var _ref8, _ref9, _object$userData, _object$userData2;

          return ((_ref8 = (_ref9 = (_object$userData = object.userData) === null || _object$userData === void 0 ? void 0 : _object$userData.parentId) !== null && _ref9 !== void 0 ? _ref9 : (_object$userData2 = object.userData) === null || _object$userData2 === void 0 ? void 0 : _object$userData2.id) !== null && _ref8 !== void 0 ? _ref8 : object.id) === id;
        });

        if (object) {
          var _id = _this5.getHitId({
            object: object
          });

          object.traverse(function (child) {
            return child.material && types.indexOf(child.type) !== -1 && child.mapMaterial(function (childMat) {
              if (!childMat.meta) return;

              var previous = _this5.selected.get(_id);

              _this5.add(_id, {
                targets: uniq((previous ? previous.targets : []).concat([target])),
                object: object
              });
            });
          });
        }
      });
    } else {
      ids.forEach(function (id) {
        var hit = target.findMaterial(function (material) {
          var _ref10, _material$meta2, _material$meta3;

          return ((_ref10 = material === null || material === void 0 ? void 0 : (_material$meta2 = material.meta) === null || _material$meta2 === void 0 ? void 0 : _material$meta2.nestedId) !== null && _ref10 !== void 0 ? _ref10 : material === null || material === void 0 ? void 0 : (_material$meta3 = material.meta) === null || _material$meta3 === void 0 ? void 0 : _material$meta3.id) === id;
        });

        if (hit) {
          var object = hit.object,
              material = hit.material;

          if (_this5.interfaceSelectors.length) {
            _this5.interfaceSelectors.forEach(function (selector) {
              var target = object.findReverse(function (parent) {
                return parent.type === selector;
              });
              target && target.traverse(function (child) {
                return child.material && types.indexOf(child.type) !== -1 && child.mapMaterial(function (childMat) {
                  if (!childMat.meta) return;

                  var previous = _this5.selected.get(childMat.meta.id);

                  if (childMat === material) {
                    _this5.add(material.meta.id, {
                      targets: uniq((previous ? previous.targets : []).concat([target])),
                      object: object,
                      material: material
                    });
                  }
                });
              });
            });
          } else if (properties[object.type]) {
            var _id2 = _this5.getHitId(hit);

            _this5.add(_id2, {
              object: object,
              material: material
            });
          }
        }
      });
    }

    return this;
  };

  _proto.target = function target(where) {
    if (where === void 0) {
      where = this.originalTarget;
    }

    this.options.target = where;
    return this;
  }; // Visually selects all items in this.selected


  _proto.selectAll = function selectAll() {
    var _this6 = this;

    var _this$options6 = this.options,
        duration = _this$options6.duration,
        properties = _this$options6.properties,
        types = _this$options6.types;
    this.selected.forEach(function (_ref11) {
      var targets = _ref11.targets,
          object = _ref11.object,
          material = _ref11.material;

      if (material === undefined) {
        object.traverse(function (child) {
          return child.material && types.indexOf(child.type) !== -1 && child.animate(child.mapMaterial(function (childMat) {
            var _properties$object$ty3;

            if (!childMat.meta) return;
            return childMat.meta.clicked = _objectSpread({}, childMat.meta.clicked, (_properties$object$ty3 = properties[object.type]) === null || _properties$object$ty3 === void 0 ? void 0 : _properties$object$ty3[CLICKED]);
          })).start(duration);
        });
      } else if (targets) {
        targets.forEach(function (target) {
          return target.traverse(function (child) {
            child.material && types.indexOf(child.type) !== -1 && child.animate(child.mapMaterial(function (childMat) {
              var _properties3;

              if (!childMat.meta) return;

              var clicked = childMat === material || _this6.selected.has(childMat.meta.id);

              return childMat.meta.clicked = _objectSpread({}, childMat.meta.clicked, (_properties3 = properties[clicked ? object.type : target.type]) === null || _properties3 === void 0 ? void 0 : _properties3[CLICKED]);
            })).start(duration);
          });
        });
      } else {
        var _properties$object$ty4;

        material.meta && material.animate(material.meta.clicked = (_properties$object$ty4 = properties[object.type]) === null || _properties$object$ty4 === void 0 ? void 0 : _properties$object$ty4[CLICKED]).start(duration);
      }
    });
    this.emit('selectAll');
    return this;
  };

  // If onlyVisual just change color, without removing from this.selected
  _proto.unselectAll = function unselectAll(onlyVisual) {
    var _this7 = this;

    if (onlyVisual === void 0) {
      onlyVisual = false;
    }

    var _this$options7 = this.options,
        target = _this$options7.target,
        duration = _this$options7.duration;
    if (this.interfaceSelectors.length) Array.from(this.map).map(function (_ref12) {
      var child = _ref12[0];
      return child;
    }).forEach(this._unselectChild);else {
      target.traverse(this._unselectChild); // Sketcher didn't contains dimensions,
      // so if target === sketcher, we have to unselect dimensions directly

      this.selected.forEach(function (el) {
        if (!el.material || !el.material.meta || !el.object) return;

        if (!_this7.unselectedObjects.has(el.object.uuid)) {
          _this7._unselectChild(el.object);

          _this7.unselectedObjects.set(el.object.uuid);
        }
      });
      this.unselectedObjects.clear();
    }
    this.emit('unselectAll');
    !onlyVisual && this.removeAll();
    return this;
  };

  _proto.getHitId = function getHitId(_ref13) {
    var _ref14, _ref15, _object$userData3, _object$userData4, _ref16, _material$meta4, _material$meta5;

    var object = _ref13.object,
        material = _ref13.material;
    return this.options.selectObjects ? (_ref14 = (_ref15 = (_object$userData3 = object.userData) === null || _object$userData3 === void 0 ? void 0 : _object$userData3.parentId) !== null && _ref15 !== void 0 ? _ref15 : (_object$userData4 = object.userData) === null || _object$userData4 === void 0 ? void 0 : _object$userData4.id) !== null && _ref14 !== void 0 ? _ref14 : object.id : (_ref16 = material === null || material === void 0 ? void 0 : (_material$meta4 = material.meta) === null || _material$meta4 === void 0 ? void 0 : _material$meta4.nestedId) !== null && _ref16 !== void 0 ? _ref16 : material === null || material === void 0 ? void 0 : (_material$meta5 = material.meta) === null || _material$meta5 === void 0 ? void 0 : _material$meta5.id;
  }; // Adds selection to the logical map (this.selected)


  _proto.add = function add(id, props, notify) {
    if (notify === void 0) {
      notify = true;
    }

    this.selected.set(id, props);
    this.last = id;
    notify && this.emit('add', id, props);
    this.handleCenterPointSelection(props);
    return this;
  }; // Removes selection from the logical map (this.selected)


  _proto.remove = function remove(id, notify) {
    if (notify === void 0) {
      notify = true;
    }

    this.selected.delete(id);
    notify && this.emit('remove', id);
    return this;
  }; // Removes all selections from the logical map (this.selected)


  _proto.removeAll = function removeAll(notify) {
    if (notify === void 0) {
      notify = true;
    }

    this.selected.clear();
    notify && this.emit('removeAll');
    return this;
  };

  _proto.calculateLinePrecision = function calculateLinePrecision() {
    var _this$options8 = this.options,
        target = _this$options8.target,
        types = _this$options8.types;

    if (this.view && types.indexOf('LineSegments') !== -1 && target.scene.bounds.sphere) {
      var center = this.view.getPoint2(target.scene.bounds.sphere.center.clone());
      var screenPoint1 = new THREE.Vector3(0, 0, center.z);
      var screenPoint2 = new THREE.Vector3(0, 0, center.z);
      screenPoint2.x = 2;
      var worldPoint1 = this.view.getPoint3(screenPoint1, this.view.camera, screenPoint1.z);
      var worldPoint2 = this.view.getPoint3(screenPoint2, this.view.camera, screenPoint2.z);
      this.view.interaction.raycaster.linePrecision = worldPoint1.sub(worldPoint2).length();
    }

    return this;
  };

  _proto.update = function update() {
    var _this8 = this;

    var _this$options9 = this.options,
        target = _this$options9.target,
        shift = _this$options9.shift;
    if (this.interfaceSelectors.length) target.traverse(function (object) {
      if (object.visible && object.interactive && object.material && _this8.basicSelectors.indexOf(object.type) !== -1) {
        var parents = _this8.interfaceSelectors.map(function (sel) {
          return object.findReverse(function (p) {
            return p.type === sel;
          });
        });

        _this8.map.set(object, [object].concat(parents).filter(function (a) {
          return a;
        }));
      }
    });
    return this;
  };

  _createClass(Selector, [{
    key: "selectedIds",
    get: function get() {
      return this.selected ? Array.from(this.selected).map(function (_ref17) {
        var id = _ref17[0],
            props = _ref17[1];
        return id;
      }) : [];
    }
  }, {
    key: "selectedMaterials",
    get: function get() {
      return this.selected ? Array.from(this.selected).map(function (_ref18) {
        var id = _ref18[0],
            material = _ref18[1].material;
        return material;
      }) : [];
    }
  }, {
    key: "selectedObjects",
    get: function get() {
      return this.selected ? Array.from(this.selected).map(function (_ref19) {
        var id = _ref19[0],
            object = _ref19[1].object;
        return object;
      }) : [];
    }
  }, {
    key: "selectedItems",
    get: function get() {
      return this.selected ? Array.from(this.selected).map(function (_ref20) {
        var id = _ref20[0],
            item = _ref20[1];
        return item;
      }) : [];
    }
  }]);

  return Selector;
}(Events);

export { Selector as default };