import _objectSpread from "@babel/runtime/helpers/objectSpread";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/objectWithoutPropertiesLoose";

var _Assembly, _Part, _Line, _LineSegments, _Group, _Mesh, _Region, _RegionPoint, _SketcherMesh, _DimensionHandle, _CSYSHandle, _Workaxis, _Workpoint;

import * as THREE from 'three';
import omit from 'lodash/omit';
import without from 'lodash/without';
import { actions as pluginActions } from './plugins';
import { actions as connectionActions } from './connections';
var scope = 'globals';
var HOVERED = THREE.Object3D.Events.Interaction.Hovered;
var CLICKED = THREE.Object3D.Events.Interaction.Clicked;
export var types = {
  setLocale: scope + "/setLocale",
  setActiveSelection: scope + "/setActiveSelection",
  setActiveConnection: scope + "/setActiveConnection",
  linkPlugins: scope + "/linkPlugins",
  unlinkPlugins: scope + "/unlinkPlugins",
  addResources: scope + "/addResources",
  addMessage: scope + "/addMessage",
  beginTask: scope + "/beginTask",
  finishTask: scope + "/finishTask",
  setCamera: scope + "/setCamera",
  setFocusedInput: scope + "/setFocusedInput",
  setControls: scope + "/setControls",
  setColors: scope + "/setColors",
  setColorMode: scope + "/setColorMode",
  showWarning: scope + "/showWarning"
};
export var actions = {
  setLocale: function setLocale(locale) {
    if (locale === void 0) {
      locale = 'en_US';
    }

    return {
      type: types.setLocale,
      locale: locale
    };
  },
  setActiveSelection: function setActiveSelection(activeSelection) {
    if (activeSelection === void 0) {
      activeSelection = undefined;
    }

    return {
      type: types.setActiveSelection,
      activeSelection: activeSelection
    };
  },
  addResources: function addResources(resources) {
    return {
      type: types.addResources,
      resources: resources
    };
  },
  linkPlugins: function linkPlugins(plugins) {
    return {
      type: types.linkPlugins,
      plugins: plugins
    };
  },
  unlinkPlugins: function unlinkPlugins(plugins) {
    return {
      type: types.unlinkPlugins,
      plugins: plugins
    };
  },
  addMessage: function addMessage(type, message, reset) {
    var _ref;

    return _ref = {
      type: types.addMessage
    }, _ref["type"] = type, _ref.message = message, _ref.reset = reset, _ref;
  },
  beginTask: function beginTask(id, command) {
    return {
      type: types.beginTask,
      id: id,
      command: command
    };
  },
  finishTask: function finishTask(id) {
    return {
      type: types.finishTask,
      id: id
    };
  },
  setActiveConnection: function setActiveConnection(activeConnection) {
    if (activeConnection === void 0) {
      activeConnection = undefined;
    }

    return function (dispatch) {
      dispatch(pluginActions.closeAllFeatures());
      activeConnection && dispatch(connectionActions.setActiveFeature(activeConnection, undefined));
      dispatch(actions.setActiveSelection());
      dispatch({
        type: types.setActiveConnection,
        activeConnection: activeConnection
      });
    };
  },
  setFocusedInput: function setFocusedInput(focusedInput) {
    return {
      type: types.setFocusedInput,
      focusedInput: focusedInput
    };
  },
  setCamera: function setCamera(camera) {
    return {
      type: types.setCamera,
      camera: camera
    };
  },
  setControls: function setControls(controls) {
    return {
      type: types.setControls,
      controls: controls
    };
  },
  setColors: function setColors(values) {
    return {
      type: types.setColors,
      values: values
    };
  },
  setColorMode: function setColorMode(day) {
    return {
      type: types.setColorMode,
      day: day
    };
  },
  showWarning: function showWarning(warningMessage) {
    return {
      type: types.showWarning,
      warningMessage: {
        msg: warningMessage,
        timeStamp: Date.now()
      }
    };
  }
};
var originalState = {
  locale: 'en_US',
  url: '',
  activeConnection: undefined,
  activeSelection: undefined,
  focusedInput: undefined,
  tasks: {},
  resources: {},
  plugins: [],
  day: true,
  colors: {
    hue: 0,
    bg: '#efefef',
    primary: '#c23369',
    accent: '#28d79f',
    text: '#373737'
  },
  selection: {
    Assembly: (_Assembly = {}, _Assembly[HOVERED] = {
      opacity: 1
    }, _Assembly[CLICKED] = {
      opacity: 1
    }, _Assembly),
    Part: (_Part = {}, _Part[HOVERED] = {
      color: new THREE.Color('#28d79f'),
      opacity: 1
    }, _Part[CLICKED] = {
      color: new THREE.Color('#c9366e'),
      opacity: 1
    }, _Part),
    Line2: (_Line = {}, _Line[HOVERED] = {
      linewidth: 10
    }, _Line[CLICKED] = {
      linewidth: 10
    }, _Line),
    LineSegments: (_LineSegments = {}, _LineSegments[HOVERED] = {
      color: new THREE.Color('#28d79f'),
      opacity: 1
    }, _LineSegments[CLICKED] = {
      color: new THREE.Color('#28b4d7'),
      opacity: 1
    }, _LineSegments),
    Group: (_Group = {}, _Group[HOVERED] = {
      color: new THREE.Color('#28d79f'),
      opacity: 1
    }, _Group[CLICKED] = {
      color: new THREE.Color('#c9366e'),
      opacity: 1
    }, _Group),
    Mesh: (_Mesh = {}, _Mesh[HOVERED] = {
      color: new THREE.Color('#28d79f'),
      opacity: 1
    }, _Mesh[CLICKED] = {
      color: new THREE.Color('#c9366e'),
      opacity: 1
    }, _Mesh),
    Region: (_Region = {}, _Region[HOVERED] = {
      color: new THREE.Color('#28d79f'),
      opacity: 1
    }, _Region[CLICKED] = {
      color: new THREE.Color('#28b4d7'),
      opacity: 1
    }, _Region),
    RegionPoint: (_RegionPoint = {}, _RegionPoint[HOVERED] = {
      color: new THREE.Color('#28d79f'),
      opacity: 1
    }, _RegionPoint[CLICKED] = {
      color: new THREE.Color('#28b4d7'),
      opacity: 1
    }, _RegionPoint),
    SketcherMesh: (_SketcherMesh = {}, _SketcherMesh[HOVERED] = {
      color: new THREE.Color('#28d79f'),
      opacity: 1,
      polygonOffsetFactor: -10,
      polygonOffsetUnits: -50
    }, _SketcherMesh[CLICKED] = {
      color: new THREE.Color('#c9366e'),
      opacity: 1,
      polygonOffsetFactor: -5,
      polygonOffsetUnits: -25
    }, _SketcherMesh),
    DimensionHandle: (_DimensionHandle = {}, _DimensionHandle[HOVERED] = {
      color: new THREE.Color('#28d79f'),
      opacity: 1
    }, _DimensionHandle[CLICKED] = {
      color: new THREE.Color('#c9366e'),
      opacity: 1
    }, _DimensionHandle),
    CSYSHandle: (_CSYSHandle = {}, _CSYSHandle[HOVERED] = {
      color: new THREE.Color('#28d79f'),
      opacity: 1
    }, _CSYSHandle[CLICKED] = {
      color: new THREE.Color('#c9366e'),
      opacity: 1
    }, _CSYSHandle),
    Workaxis: (_Workaxis = {}, _Workaxis[HOVERED] = {
      color: new THREE.Color('#28d79f')
    }, _Workaxis[CLICKED] = {
      color: new THREE.Color('#c9366e')
    }, _Workaxis),
    Workpoint: (_Workpoint = {}, _Workpoint[HOVERED] = {
      color: new THREE.Color('#28d79f')
    }, _Workpoint[CLICKED] = {
      color: new THREE.Color('#c9366e')
    }, _Workpoint)
  },
  camera: 'orthographic',
  controls: 'trackball',
  warningMessage: ''
};
export var reducer = function reducer(state, _ref2) {
  var _objectSpread2;

  if (state === void 0) {
    state = originalState;
  }

  var type = _ref2.type,
      payload = _objectWithoutPropertiesLoose(_ref2, ["type"]);

  switch (type) {
    case types.setLocale:
    case types.setActiveSelection:
    case types.setActiveConnection:
    case types.setControls:
    case types.showWarning:
    case types.setCamera:
    case types.setFocusedInput:
      return _objectSpread({}, state, payload);

    case types.setColorMode:
      return _objectSpread({}, state, payload);
      break;

    case types.setColors:
      return _objectSpread({}, state, {
        colors: _objectSpread({}, state.colors, payload.values)
      });

    case types.addResources:
      return _objectSpread({}, state, {
        resources: _objectSpread({}, state.resources, payload.resources)
      });

    case types.linkPlugins:
      return _objectSpread({}, state, {
        plugins: state.plugins.concat(Array.isArray(payload.plugins) ? payload.plugins : [payload.plugins])
      });

    case types.unlinkPlugins:
      return _objectSpread({}, state, {
        plugins: without.apply(void 0, [state.plugins].concat(Array.isArray(payload.plugins) ? payload.plugins : [payload.plugins]))
      });

    case types.beginTask:
      return _objectSpread({}, state, {
        tasks: _objectSpread({}, state.tasks, (_objectSpread2 = {}, _objectSpread2[payload.id] = payload.command, _objectSpread2))
      });

    case types.finishTask:
      return _objectSpread({}, state, {
        tasks: omit(state.tasks, payload.id)
      });

    default:
      return state;
  }
};